import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { User } from '../core/user';

import { CashierReport } from './cashier-report';

const url = '/api/cashier-report';
const serviceName = 'CashierReportService';

@Injectable({
  providedIn: 'root',
})
export class CashierReportService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(id: string, startDate: string, finishDate): Observable<CashierReport> {
    const listUrl = id === null ? url : `${url}/${id}`;
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<CashierReport>>(
      this.http
        .get<CashierReport>(listUrl, options)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  activeCashiers(startDate: string, finishDate): Observable<User[]> {
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<User[]>>this.http
      .get<User[]>(`${url}/active`, options)

      .pipe(catchError(this.log.handleError(serviceName, 'activeCashiers')));
  }

  print(id: string, startDate: string, finishDate): Observable<boolean> {
    const printUrl = `${url}/print/${id}`;
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<boolean>>(
      this.http
        .get<boolean>(printUrl, options)
        .pipe(catchError(this.log.handleError(serviceName, 'print')))
    );
  }
}
