import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { ModifierCategory } from '../core/modifier-category';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/modifier-categories';
const serviceName = 'ModifierCategoryService';

@Injectable({
  providedIn: 'root',
})
export class ModifierCategoryService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string): Observable<ModifierCategory> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return <Observable<ModifierCategory>>(
      this.http
        .get<ModifierCategory>(getUrl)
        .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`)))
    );
  }

  list(): Observable<ModifierCategory[]> {
    return <Observable<ModifierCategory[]>>(
      this.http
        .get<ModifierCategory[]>(`${url}/list`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  listForProduct(id: string): Observable<ModifierCategory[]> {
    return <Observable<ModifierCategory[]>>(
      this.http
        .get<ModifierCategory[]>(`${url}/for-product/${id}`)
        .pipe(catchError(this.log.handleError(serviceName, 'listIsActiveOfProduct')))
    );
  }

  save(modifierCategory: ModifierCategory): Observable<ModifierCategory> {
    return <Observable<ModifierCategory>>(
      this.http
        .post<ModifierCategory>(url, modifierCategory, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'save')))
    );
  }

  update(modifierCategory: ModifierCategory): Observable<ModifierCategory> {
    return <Observable<ModifierCategory>>(
      this.http
        .put<ModifierCategory>(`${url}/${modifierCategory.id}`, modifierCategory, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  saveOrUpdate(modifierCategory: ModifierCategory): Observable<ModifierCategory> {
    if (!modifierCategory.id) {
      return this.save(modifierCategory);
    }
    return this.update(modifierCategory);
  }

  delete(id: string): Observable<ModifierCategory> {
    return <Observable<ModifierCategory>>(
      this.http
        .delete<ModifierCategory>(`${url}/${id}`, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'delete')))
    );
  }
}
