import { DataSource } from '@angular/cdk/collections';
import { merge, Observable } from 'rxjs';
import { map, tap } from 'rxjs/operators';

import { Product } from '../../core/product';

export class ProductListDataSource extends DataSource<Product> {
  public data: Product[];
  public viewData: Product[];
  private filterValue: string;

  constructor(
    private readonly filter: Observable<string>,
    private readonly dataObs: Observable<Product[]>,
  ) {
    super();
    this.data = [];
    this.viewData = [];
    this.filter = filter.pipe(
      tap((x) => {
        this.filterValue = x;
      }),
    );
    this.dataObs = dataObs.pipe(
      tap((x) => {
        this.data = x;
      }),
    );
  }

  connect(): Observable<Product[]> {
    const dataMutations = [this.dataObs, this.filter];

    return merge(...dataMutations).pipe(
      map(() => {
        this.viewData = this.getFilteredData([...this.data]);
        return this.viewData;
      }),
    );
  }

  disconnect() {}

  private getFilteredData(data: Product[]): Product[] {
    const filter = this.filterValue === undefined ? '' : this.filterValue;
    if (filter === '') {
      return data;
    }
    return data.filter((x) => {
      return x.menuCategory.id === filter;
    });
  }
}
