import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { SaleCategory } from '../core/sale-category';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/sale-categories';
const serviceName = 'SaleCategoryService';

@Injectable({
  providedIn: 'root',
})
export class SaleCategoryService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string): Observable<SaleCategory> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return <Observable<SaleCategory>>(
      this.http
        .get<SaleCategory>(getUrl)
        .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`)))
    );
  }

  list(): Observable<SaleCategory[]> {
    return <Observable<SaleCategory[]>>(
      this.http
        .get<SaleCategory[]>(`${url}/list`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  listForDiscount(): Observable<{ name: string; discount: number }[]> {
    return <Observable<{ name: string; discount: number }[]>>(
      this.http
        .get<{ name: string; discount: number }[]>(`${url}/for-discount`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  save(saleCategory: SaleCategory): Observable<SaleCategory> {
    return <Observable<SaleCategory>>(
      this.http
        .post<SaleCategory>(url, saleCategory, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'save')))
    );
  }

  update(saleCategory: SaleCategory): Observable<SaleCategory> {
    return <Observable<SaleCategory>>(
      this.http
        .put<SaleCategory>(`${url}/${saleCategory.id}`, saleCategory, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  saveOrUpdate(saleCategory: SaleCategory): Observable<SaleCategory> {
    if (!saleCategory.id) {
      return this.save(saleCategory);
    }
    return this.update(saleCategory);
  }

  delete(id: string): Observable<SaleCategory> {
    return <Observable<SaleCategory>>(
      this.http
        .delete<SaleCategory>(`${url}/${id}`, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'delete')))
    );
  }
}
